﻿/* 
    Copyright (c) Microsoft Corporation. All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
  
    To see all code Samples for Windows Store apps and Windows Phone Store apps, visit http://code.msdn.microsoft.com/windowsapps
  
*/

namespace Bookstore3Universal
{
	using SdkHelper;
	using System;
	using System.Collections.Generic;
	using System.Collections.ObjectModel;
	using System.Globalization;
	using System.Linq;
	using Windows.ApplicationModel;
	using Windows.UI.Xaml;
	using Windows.UI.Xaml.Media;
	using Windows.UI.Xaml.Media.Imaging;

	public class SortedEnGrouping
	{
		private ReadOnlyCollection<string> groupDisplayNames;

		//
		// Summary:
		//     Get the strings for all the groups for the En locale.
		//
		// Returns:
		//     The strings for all the groups for the En locale.
		public ReadOnlyCollection<string> GroupDisplayNames
		{
			get
			{
				if (this.groupDisplayNames == null)
				{
					List<string> readWriteGroupDisplayNames = new List<string>();
					readWriteGroupDisplayNames.Add("#");
					for (char letter = 'a'; letter <= 'z'; ++letter)
					{
						readWriteGroupDisplayNames.Add(letter.ToString());
					}
					readWriteGroupDisplayNames.Add("...");
					this.groupDisplayNames = new ReadOnlyCollection<string>(readWriteGroupDisplayNames);
				}
				return this.groupDisplayNames;
			}
		}

		// Summary:
		//     Returns the group index to which the value string belongs.
		//
		// Parameters:
		//   itemName:
		//     The specified string.
		//
		// Returns:
		//     The group index to which the value string belongs.
		public int GetGroupIndex(string itemName)
		{
			if (itemName.Length > 0)
			{
				return groupDisplayNames.IndexOf(char.ToLower(itemName[0]).ToString());
			}
			else
			{
				return -1;
			}
		}
	}

	public class GroupByHelper<T> : List<T>
	{
		/// <summary>
		/// The delegate that is used to get the key information.
		/// </summary>
		/// <param name="item">An object of type T</param>
		/// <returns>The key value to use for this object</returns>
		public delegate string GetKeyDelegate(T item);

		/// <summary>
		/// The Key of this group.
		/// </summary>
		public string Key { get; private set; }

		/// <summary>
		/// The has-items Visibility of this group.
		/// </summary>
		public Visibility GroupHasItemsVisibility
		{
			get
			{
				return (this.Count == 0) ? Visibility.Collapsed : Visibility.Visible;
			}
		}

		/// <summary>
		/// Public constructor.
		/// </summary>
		/// <param name="key">The key for this group.</param>
		public GroupByHelper(string key)
		{
			this.Key = key;
		}

		public GroupByHelper(IGrouping<string, T> grouping)
			: base(grouping)
		{
			this.Key = grouping.Key;
		}

		/// <summary>
		/// Create a list of GroupByHelper<T> grouped by keys set by a SortedEnGrouping.
		/// </summary>
		/// <param name="items">The items to place in the groups.</param>
		/// <param name="getKey">A delegate to get the key from an item.</param>
		/// <param name="sort">Will sort the data if true.</param>
		/// <returns>An items source for a ListView or GridView</returns>
		public static List<GroupByHelper<T>> CreateSortedEnGroupingByInitial(IEnumerable<T> listOfItems, GetKeyDelegate getKey, bool sort)
		{
			// Create a list of GroupByHelper<T> with keys set by a SortedEnGrouping.
			SortedEnGrouping sortedEnGrouping = new SortedEnGrouping();
			List<GroupByHelper<T>> listOfGroups = new List<GroupByHelper<T>>();
			foreach (string key in sortedEnGrouping.GroupDisplayNames)
			{
				listOfGroups.Add(new GroupByHelper<T>(key));
			}

			foreach (T item in listOfItems)
			{
				int index = 0;
				index = sortedEnGrouping.GetGroupIndex(getKey(item));
				if (index >= 0 && index < listOfGroups.Count)
				{
					listOfGroups[index].Add(item);
				}
			}

			if (sort)
			{
				foreach (GroupByHelper<T> group in listOfGroups)
				{
					group.Sort((c0, c1) => { return CultureInfo.CurrentUICulture.CompareInfo.Compare(getKey(c0), getKey(c1)); });
				}
			}

			return listOfGroups;
		}
	}

	public class BookstoreViewModel : PropertyChangedNotifier
	{
		#region fields
		private RelayCommand atticusCommand;
		private List<GroupByHelper<BookSku>> authors;
		private ObservableCollection<BookSku> bookSkus;
		private IOrderedEnumerable<IGrouping<string, BookSku>> genres;
		private bool hasAtticusBeenUpdated = false;
		private bool isMobyDickInTheStore = false;
		private RelayCommand mobyDickCommand;
		#endregion fields

		#region properties
		public string AppName
		{
			get
			{
				return "BOOKSTORE3UNIVERSAL";
			}
		}

		public List<GroupByHelper<BookSku>> Authors
		{
			get
			{
				if (this.authors == null)
				{
					this.authors = GroupByHelper<BookSku>.CreateSortedEnGroupingByInitial(this.bookSkus,
									(BookSku bookSku) => { return bookSku.authorName; }, true);
				}
				return this.authors;
			}
		}

		public ObservableCollection<BookSku> BookSkus { get { return this.bookSkus; } }

		private bool CanAddMobyDick
		{
			get
			{
				return !isMobyDickInTheStore;
			}
		}

		private bool CanUpdateAtticus
		{
			get
			{
				return !hasAtticusBeenUpdated;
			}
		}

		public IOrderedEnumerable<IGrouping<string, BookSku>> Genres
		{
			get
			{
				if (this.genres == null)
				{
					this.genres = from book in this.bookSkus group book by book.genre into grp orderby grp.Key select grp;
				}
				return this.genres;
			}
		}

		public Visibility CollapsedIfWindowsApp
		{
			get
			{
#if WINDOWS_APP
				return Visibility.Collapsed;
#else
				return Visibility.Visible;
#endif // WINDOWS_APP
			}
		}
		#endregion properties

		#region methods
		private void AddMobyDick()
		{
			this.bookSkus.Add(new BookSku("Moby-Dick", "Melville, Herman", "Adventure", "/Assets/CoverImages/two.png"));
			this.isMobyDickInTheStore = true;
			this.MobyDickCommand.RaiseCanExecuteChanged();
			this.authors = null;
			this.RaisePropertyChanged("Authors");
			this.genres = null;
			this.RaisePropertyChanged("Genres");
		}

		private void UpdateAtticus()
		{
			foreach (BookSku bookSku in this.bookSkus)
			{
				if (bookSku.Title == "Atticus")
				{
					bookSku.Title = "To Kill a Mockingbird";
					this.hasAtticusBeenUpdated = true;
					this.AtticusCommand.RaiseCanExecuteChanged();
					break;
				}
			}
		}
		#endregion methods

		#region constructors
		public BookstoreViewModel()
		{
			// Establish the invariant of owning a collection of BookSku.
			this.bookSkus = new ObservableCollection<BookSku>();

			if (DesignMode.DesignModeEnabled)
			{
				DataSource.LoadSampleData(ref this.bookSkus);
			}
			else
			{
				DataSource.LoadDataFromCloudService(ref this.bookSkus);
			}
		}
		#endregion constructors

		#region commands
		/// <summary>
		/// You can bind the Command property of a Button to a property of type RelayCommand. When the Button is clicked, the command will be
		/// invoked. The Button will be enabled as long as the command can execute.
		/// </summary>
		public RelayCommand AtticusCommand
		{
			get
			{
				if (this.atticusCommand == null)
				{
					// The RelayCommand is constructed with two parameters - the action to perform on invocation,
					// and the condition under which the command can execute. It's important to call RaiseCanExecuteChanged
					// on a command whenever its can-execute condition might have changed. Here, we do that in the TrialExperienceHelper_LicenseChanged
					// event handler.
					this.atticusCommand = new RelayCommand(
						param => this.UpdateAtticus(),
						param => this.CanUpdateAtticus);
				}
				return this.atticusCommand;
			}
		}

		public RelayCommand MobyDickCommand
		{
			get
			{
				if (this.mobyDickCommand == null)
				{
					// The RelayCommand is constructed with two parameters - the action to perform on invocation,
					// and the condition under which the command can execute. It's important to call RaiseCanExecuteChanged
					// on a command whenever its can-execute condition might have changed. Here, we do that in the TrialExperienceHelper_LicenseChanged
					// event handler.
					this.mobyDickCommand = new RelayCommand(
						param => this.AddMobyDick(),
						param => this.CanAddMobyDick);
				}
				return this.mobyDickCommand;
			}
		}
		#endregion commands
	}

	public class BookSku : PropertyChangedNotifier
	{
		#region fields
		internal string authorName;
		internal string genre;
		private string title;
		#endregion fields

		#region properties
		public string AuthorNameAndGenre
		{
			get
			{
				return this.authorName + " | " + this.genre;
			}
		}

		public ImageSource CoverImage
		{
			get
			{
				// this.CoverImagePath contains a path of the form "/Assets/CoverImages/one.png".
				return new BitmapImage(new Uri(new Uri("ms-appx://"), this.CoverImagePath));
			}
		}

		public string CoverImagePath { get; private set; }

		public string Title
		{
			get
			{
				return this.title;
			}
			internal set
			{
				this.SetPropertyAndRaisePropertyChanged(ref this.title, value);
			}
		}
		#endregion properties

		#region constructors
		public BookSku(string title, string authorName, string genre, string coverImagePath)
		{
			this.Title = title;
			this.authorName = authorName;
			this.genre = genre;
			this.CoverImagePath = coverImagePath;
		}
		#endregion constructors
	}

	internal static class DataSource
	{
		#region methods
		public static void LoadSampleData(ref ObservableCollection<BookSku> bookSkus)
		{
			bookSkus.Add(new BookSku("Emma", "Austen, Jane", "Romance", "/Assets/CoverImages/three.png"));
			bookSkus.Add(new BookSku("Mansfield Park", "Austen, Jane", "Romance", "/Assets/CoverImages/six.png"));
			bookSkus.Add(new BookSku("Persuasion", "Austen, Jane", "Romance", "/Assets/CoverImages/four.png"));
			bookSkus.Add(new BookSku("Pride and Prejudice", "Austen, Jane", "Romance", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("Sense and Sensibility", "Austen, Jane", "Romance", "/Assets/CoverImages/seven.png"));
			bookSkus.Add(new BookSku("The Secret Garden", "Burnett, Frances Hodgson", "Children's", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("A Christmas Carol", "Dickens, Charles", "Fantasy", "/Assets/CoverImages/one.png"));
			bookSkus.Add(new BookSku("Atticus", "Lee, Harper", "Drama", "/Assets/CoverImages/four.png"));
			bookSkus.Add(new BookSku("David Copperfield", "Dickens, Charles", "Drama", "/Assets/CoverImages/three.png"));
			bookSkus.Add(new BookSku("East of Eden", "Steinbeck, John", "Drama", "/Assets/CoverImages/two.png"));
			bookSkus.Add(new BookSku("Gone With The Wind", "Mitchell, Margaret", "Historical", "/Assets/CoverImages/four.png"));
			bookSkus.Add(new BookSku("Great Expectations", "Dickens, Charles", "Drama", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("Little Dorrit", "Dickens, Charles", "Drama", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("Oliver Twist", "Dickens, Charles", "Drama", "/Assets/CoverImages/two.png"));
			bookSkus.Add(new BookSku("Treasure Island", "Stevenson, R.L.", "Adventure", "/Assets/CoverImages/five.png"));
			bookSkus.Add(new BookSku("The Fellowship Of The Ring", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/one.png"));
			bookSkus.Add(new BookSku("The Hobbit", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/one.png"));
			bookSkus.Add(new BookSku("The Return Of The King", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/two.png"));
			bookSkus.Add(new BookSku("The Two Towers", "Tolkien, J.R.R.", "Fantasy", "/Assets/CoverImages/six.png"));
		}

		public static void LoadDataFromCloudService(ref ObservableCollection<BookSku> bookSkus)
		{
			// In this simple app, we'll simulate real-world data access by loading sample data.
			DataSource.LoadSampleData(ref bookSkus);
		}
		#endregion methods
	}
}